/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2013.07.27.
*   Chip         -  Atmel ATmega8
*   Compiler     -  avr-gcc (WinAVR)
*
*   KT SIRC vevo
*   
********************************************************************************
*
*   AVR  4 - INT0 - TSOP 2236 2
*   AVR  1 - NRES - switch, 4k7-tel felhuzva
*   AVR 15 - PB1  - LED, 470R-el felhuzva
*
*******************************************************************************/

#define F_CPU                4 MHZ
#define T0_DIVIDER           64        // 16 usec




#include "tkiraaly_atmega8.h"
#include <util/delay.h>
#include <avr/pgmspace.h>




volatile U8 ir_flag;                   // adat kesz
volatile U8 ir_bitcount;               // bitek szamlalasa
volatile U8 ir_command;                // 7 bit - parancs kod
volatile U8 ir_command_;
volatile U8 ir_device;                 // 5/8 bit - eszkoz kod
volatile U8 ir_device_;
//          ir_extend                  // 0/8 bit - nincs hasznalva  




void usart_putc( U8);                  // USART 1 karakter kiirasa
void usart_puthexc( U8);               // USART 1 hexadecimalis szamjegy kiirasa
void usart_puthexb( U8);               // USART 1 byte hexadecimalis kiirasa




ISR( INT0_vect)                        // INT0 - felfuto el figyelese
{
   T0_STOP;
   if( TCNT0 < 200)                    // bit/ nem adas koz
   {
      if ( ir_bitcount < 16)           // csak az elso 15 bit feldolgozasa
      {
         if ( ir_bitcount < 8)         // command bitek (7)
         { 
            ir_command_ >>= 1;
            if( TCNT0 > 90) ir_command_ |= 0B01000000;  // 1
         }
         else
         {                             // device bitek (8)
            ir_device_ >>= 1;
            if( TCNT0 > 90) ir_device_ |= 0B10000000;   // 1
         }
         ir_bitcount++;
      }
   }
   TCNT0= 0;                            // meres inditasa
   T0_NORMAL;
}




ISR( TIMER0_OVF_vect)                  // 255 periodusnal hosszabb ido, adas vege
{
   T0_STOP;
   TCNT0= 255;                         // atporges visszaallitas
   ir_command= ir_command_;            // adatok kimasolasa
   ir_device= ir_device_;
   ir_flag= 1;                         // flag beallitas
   ir_bitcount= 1;                     // valtozok alapba allitasa
   ir_command_= 0;
   ir_device_= 0;
}




int main( void)
{
   U8 t_command;
   U8 t_device;

   PB1_OUT;                            // LED

   T0_STOP;
   TCNT0= 255;
   T0_IT_OVF_ENABLE;

   ir_flag= 0;
   ir_bitcount= 1;
   ir_command_= 0;
   ir_device_= 0;

   INT0_RISING;
   IT_INT0_ENABLE;

   USART_SET_9600_8N1;

   IT_ENABLE;

   usart_putc('S');
   usart_putc('I');
   usart_putc('R');
   usart_putc('C');
   usart_putc( 13);
   
   for(;;)
   {
      if( ir_flag)
      {
         t_command= ir_command;
         t_device=  ir_device;
         ir_flag= 0;
         usart_puthexb( t_command);
         usart_putc( '-');
         usart_puthexb( t_device);
         usart_putc( 13);
      } 
   }
   return 0;
}

     


void usart_putc( U8 c)                 // USART 1 karakter kiiras
{
    while( BTC( UCSRA, UDRE));         // 1, ha kesz az adat fogadasara
    UDR= c;
}



 
void usart_puthexc( U8 c)              // USART 1 hexadecimalis szamjegy kiirasa
{
   c&= 0x0F;
   if( c < 10) c+= '0';
   else        c+= 'A'- 10;
   usart_putc( c);
}




void usart_puthexb( U8 c)              // USART 1 byte hexadecimalis kiirasa
{
   usart_puthexc( c >> 4);
   usart_puthexc( c);
}
